﻿using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using Windows.UI.Popups;
using Windows.UI.Xaml.Documents;
using Windows.UI.Xaml.Markup;

namespace RichTextDemo
{
    public sealed partial class MainPage
    {
        //表情字典，表情转文件名
        private readonly Dictionary<string, string> emojiDict = new Dictionary<string, string>
        {
            {"大笑", "daxiao"},
            {"大哭", "daku"},
        };

        public MainPage()
        {
            InitializeComponent();
            Loaded += MainPage_Loaded;
        }

        private void MainPage_Loaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            
            var richText = @"2015年喽嘻嘻~~新年新气象，三羊开泰，诸事喜气洋洋。GO~，发红包啦<a href=""http://www.baidu.com"">点我领红包</a>{大笑}哈哈{大哭}呜呜";

            var p = AnalyzeText(richText);
            Rtb.Blocks.Add(p);
        }

        /// <summary>
        /// 文本转富文本
        ///     表情为格式为：{表情}
        ///     链接格式为：<a href="http://www.baidu.com">百度</a>
        /// </summary>
        private Paragraph AnalyzeText(string richText)
        {
            richText = Regex.Replace(richText, "{(?<emoji>[^}]+)}|<a[^>]*?>(?<link_title>[^<]*?)</a>", i =>
            {
                if (!string.IsNullOrEmpty(i.Groups["link_title"].Value))
                {
                    const string link =
                        @"<Hyperlink Foreground=""{{StaticResource PhoneAccentBrush}}"" NavigateUri=""http://www.baidu.com"">
                                        <Underline>{0}</Underline>
                                    </Hyperlink>";
                    return string.Format(link, i.Groups["link_title"].Value);
                }
                if (!string.IsNullOrEmpty(i.Groups["emoji"].Value))
                {
                    const string image = @"<InlineUIContainer>
                                            <Image Source=""/Assets/Emoji/{0}.png"" Width=""30"" VerticalAlignment=""Center""/>
                                        </InlineUIContainer>";
                    return string.Format(image, emojiDict[i.Groups["emoji"].Value]);
                }
                
                return string.Empty;
            });

            const string paragraph = @"<Paragraph 
                            xmlns=""http://schemas.microsoft.com/winfx/2006/xaml/presentation""
                            xmlns:x=""http://schemas.microsoft.com/winfx/2006/xaml"">
                            {0}
                        </Paragraph>";

            var p = (Paragraph)XamlReader.Load(string.Format(paragraph, richText));

            HandleHyperlink(p);
            return p;
        }

        private void Test1()
        {
            var text = @"拉拉拉<a href=""http://www.baidu.com"">百度</a>拉拉{大笑}表情哈哈{大哭}";

            text = Regex.Replace(text, "{(?<emoji>[^}]+)}", i =>
            {
                const string image = @"<InlineUIContainer>
                                        <Image Source=""/Assets/Logo.png"" Width=""30"" VerticalAlignment=""Center""/>
                                    </InlineUIContainer>";
                return string.Format(image);
            });

            text = Regex.Replace(text, "<a[^>]*?>(?<title>[^<]*?)</a>", i =>
            {
                const string link =
                    @"<Hyperlink Foreground=""{{StaticResource PhoneAccentBrush}}"" NavigateUri=""http://www.baidu.com"">
                                        <Underline>{0}</Underline>
                                    </Hyperlink>";
                return string.Format(link, i.Groups["title"].Value);
            });



            const string paragraph = @"<Paragraph 
                            xmlns=""http://schemas.microsoft.com/winfx/2006/xaml/presentation""
                            xmlns:x=""http://schemas.microsoft.com/winfx/2006/xaml"">
                            {0}
                        </Paragraph>";

            var p = (Paragraph)XamlReader.Load(string.Format(paragraph, text));

            HandleHyperlink(p);
            Rtb.Blocks.Add(p);
        }

        /// <summary>
        /// 处理点击事件
        /// </summary>
        /// <param name="p"></param>
        private void HandleHyperlink(Paragraph p)
        {
            foreach (var inline in p.Inlines)
            {
                var link = inline as Hyperlink;
                if (link != null)
                {
                    HyperLinkExtensions.SetNavigateUrl(link, link.NavigateUri.OriginalString);
                    link.NavigateUri = null;
                    link.Click += link_Click;
                }
            }
        }

        private async void link_Click(Hyperlink sender, HyperlinkClickEventArgs args)
        {
            var uri = HyperLinkExtensions.GetNavigateUrl(sender);
            //这里定义点击事件
            await new MessageDialog(uri).ShowAsync();
        }
    }
}
